/*
   Widgetklasse:	XwHistbarWidgetClass
      WHistbar.c

$Revision: 2.2 $

$Log: WHistbar.c,v $
Revision 2.2  2001/11/17 12:30:36  keidel
corrected problem with fontlist at destroy

Revision 2.1  2001/02/25 14:57:15  keidel
Set all to stable version 2.0

Revision 1.1.1.1  2001/02/25 14:48:33  keidel
initial motif in motion widgets


Revision 1.7  2000/08/04 16:01:23  keidel
added black line at upper field

Revision 1.6  1998/10/01 14:37:05  keidel
Correction of Scaling, (went bad if negative scale-minimum

Revision 1.5  1998/02/09 16:23:34  keidel
final cleaning (using IRIS -fullwarn on an origin with IRIX 6.4)

Revision 1.4  1998/02/07 16:15:09  keidel
 due to -fullwarn on IRIX6.4 eliminating unused etc

Revision 1.3  1998/02/02 14:53:31  root
corrected reset if new scale or new grid

Revision 1.2  1998/01/30 17:04:17  keidel
going beta, all functions basically tested

Revision 1.1  1998/01/18 08:00:02  keidel
Initial revision


  
        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2, or (at your option)
        any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/******************************************************************
   Widgetclass:		XwHistbarWidgetClass

   written by: 	WizAnt Software
   		Jrgen Keidel  
   		Bergstrasse 15
   		D 85080 Gaimersheim
   		Tel/Fax: D  08406 379
   		E-mail: wizant.keidel@t-online.de
   		1998
                                                                  
******************************************************************/
/*	History:    initial: Mon Dec 29 13:34:28 CET 1997


*/		
/* include */

#include <stdio.h>
#include <stdlib.h>
#ifdef LINUX
#define __USE_BSD
#endif
#include <math.h>
#include  <limits.h>
#include <values.h>
#include <X11/Xlib.h>
#include<Xm/PrimitiveP.h>
#include<Xm/DrawP.h>


#include <WHistbar.h>


#ifdef __cplusplus
extern "C"{
#endif

#define offset(field) XtOffsetOf(XwHistbarRec, field)
#ifndef strdup
char *strdup(const char *s);
#endif


#define MACRO 


	
/****** private Funktions Deklaration ******/
#ifdef _NO_PROTO

/* Deklaration der Methoden */
static void Initialize();
static Boolean SetValues ();
static void Redisplay();
static XtGeometryResult QueryGeometry( );
static void Resize();
static void Destroy(  );

/* Deklaration der privaten Funktionen */
static void get_proportion();
static void Gethistbar_gc();
static void clear_histbar();
static void scale();
static void drawgrid();
static void drawbar();
static void drawannot();
static void  copyannot();
static void  copypix();
static void  dancevals();
static void  add_history ();
static void  create_history ();
static void  addhistory ();
static void select_action();
static void Gethistbarcolours();
#else

/* Deklaration der Methoden */
static void Initialize		( Widget , Widget , ArgList , Cardinal *);
static Boolean SetValues 		( Widget curr, Widget requ, Widget new, ArgList args
				, Cardinal *numargs);
static void Redisplay		(Widget w, XEvent *event, Region region);
static XtGeometryResult QueryGeometry( Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer);
static void Resize		( Widget );
static void Destroy		( Widget );

/* Deklaration der privaten Funktionen */
static void get_proportion	( XwHistbarWidget );
static void Gethistbar_gc	( XwHistbarWidget  );
static void clear_histbar	(XwHistbarWidget );
static void scale		(XwHistbarWidget);
static void drawgrid		(XwHistbarWidget );
static void drawbar		(XwHistbarWidget ,  long int id , float val , float time);
static void drawannot		(XwHistbarWidget  cw);
static void copyannot		(XwHistbarWidget  cw ,Drawable target, double yoffset);
static void copypix		(XwHistbarWidget  cw , int cop);
static void dancevals		(XwHistbarWidget  cw );
static void add_history	    	(XwHistbarWidget  cw, long int id);
static void create_history 	(XwHistbarWidget  cw);
static void addhistory		(Histbarhistory *hist, float xy[]);
static void select_action	( Widget , XEvent *);
static void Gethistbarcolours	(XwHistbarWidget  cw , ScBars  *bars);
#endif /* _NO_PROTO */


/* Definition der Resourcen-Liste */


static XtResource resources[] = {
    {
    	XmNinputCallback,
    	NULL,
    	XtRCallback,
    	sizeof(XtPointer),
    	offset(histbar.input_callback),
    	XtRCallback,
    	NULL

  },
  {
    	XmNexposeCallback,
    	XmCExposeCallback,
    	XtRCallback,
    	sizeof(XtPointer),
    	offset(histbar.expose_callback),
    	XtRCallback,
    	NULL

  },
 {
    	XmNresizeCallback,
    	XmCResizeCallback,
    	XtRCallback,
    	sizeof(XtPointer),
    	offset(histbar.resize_callback),
    	XtRCallback,
    	NULL

  },
 {
   	XwNhistbarGridScale ,
   	XwCHistbarGridScale,
  	XtRFloat ,  	  	 
  	sizeof(float *),
  	offset(histbar.gr),	
  	XtRImmediate,
  	NULL
  },
  {
   	XwNhistbarScale,
   	XwCHistbarScale,
  	XtRFloat ,  	  	 
  	sizeof(float *),
  	offset(histbar.vis),	
  	XtRImmediate,
  	NULL
  },
 {
   	XwNhistbarAnnotColor,
   	XwCHistbarAnnotColor,
  	XtRPixel ,  	  	 
  	sizeof(Pixel),
  	offset(histbar.anncolor ),	
  	XtRImmediate,
  	(XtPointer) NULL
  },
{
   	XwNhistbarBARcolor ,
   	XwCHistbarBARcolor ,
  	XtRPixel ,  	  	 
  	sizeof(Pixel),
  	offset(histbar.background ),	
  	XtRImmediate,
  	(XtPointer) NULL
  },
   {
   	XwNhistbarAnnotTNdec,
   	XwCHistbarAnnotTNdec,
  	XtRInt ,  	  	 
  	sizeof(int),
  	offset(histbar.xndecimals ),	
  	XtRImmediate,
  	(XtPointer) 0
  },
   {
   	XwNhistbarAnnotYNdec,
   	XwCHistbarAnnotYNdec,
  	XtRInt ,  	  	 
  	sizeof(int),
  	offset(histbar.yndecimals ),	
  	XtRImmediate,
  	(XtPointer) 0
  },
  {
   	XwNhistbarAnnotTNgrid,
   	XwCHistbarAnnotTNgrid,
  	XtRInt ,  	  	 
  	sizeof(int),
  	offset(histbar.xnthgridanno),	
  	XtRImmediate,
  	(XtPointer) 0
  },
   {
   	XwNhistbarAnnotYNgrid,
   	XwCHistbarAnnotYNgrid,
  	XtRInt ,  	  	 
  	sizeof(int),
  	offset(histbar.ynthgridanno),	
  	XtRImmediate,
  	(XtPointer) 0
  },
 {
   	XwNhistbarNbars,
   	XwCHistbarNbars,
  	XtRInt ,  	  	 
  	sizeof(int),
  	offset(histbar.nbars ),	
  	XtRImmediate,
  	(XtPointer) DefaultNbars
  },
  {
   	XwNhistbarAnnotation,
   	XwCHistbarAnnotation,
  	XtRInt ,  	  	 
  	sizeof(int),
  	offset(histbar.anflag ),	
  	XtRImmediate,
  	(XtPointer) (XwHistbarAnno)
  },
 {
   	XwNhistbarAddvalue,
   	XwCHistbarAddvalue,
  	XtRBool ,  	  	 
  	sizeof(int),
  	offset(histbar.addvalues),	
  	XtRImmediate,
  	(XtPointer) FALSE
  },
 {
   	XwNhistbarAddtext,
   	XwCHistbarAddtext,
  	XtRBool ,  	  	 
  	sizeof(int),
  	offset(histbar.barnames),	
  	XtRImmediate,
  	(XtPointer) FALSE
  },
 {
   	XwNhistbarBar,
   	XwCHistbarBar,
  	XtRPixel ,  	  	 
  	sizeof(Pixel),
  	offset(histbar.firstbar ),	
  	XtRImmediate,
  	(XtPointer) NULL
  },
  {
   	XwNhistbarWidth,
   	XwCHistbarWidth,
  	XtRDimension ,  	  	 
  	sizeof(Dimension),
  	offset(histbar.barwidth ),	
  	XtRImmediate,
  	(XtPointer) DefaultBarWidth
  },
  {
   	XwNhistbarSize,
   	XwCHistbarSize,
  	XtRInt ,  	  	 
  	sizeof(int),
  	offset(histbar.barfactor ),	
  	XtRImmediate,
  	(XtPointer) DefaultBarSize
  },
  
{
 	XmNfontList,
 	XmCFontList,
 	XmRFontList,
 	sizeof(XmFontList),
 	offset(histbar.font_list),
 	XtRImmediate,
 	NULL
  },
   {
  	XwNhistbarHistory,
  	XwCHistbarHistory,
  	XtRBool,
  	sizeof(int),
  	offset(histbar.history),
  	XtRImmediate,
  	(XtPointer) TRUE
  },
 {
   	XwNhistbarHistorysize,
   	XwCHistbarHistorysize,
  	XtRInt,  	  	 
  	sizeof(int),
  	offset(histbar.historysize ),	
  	XtRImmediate,
  	(XtPointer) 0
  },
 {
   	XwNhistbarAngle,
   	XwCHistbarAngle,
  	XtRInt,  	  	 
  	sizeof(int),
  	offset(histbar.Angle ),	
  	XtRImmediate,
  	(XtPointer) XwHistbarDefaultAngle
  },
};
/*Definition der Translation Tabelle */

static char defaultTranslations[] =
  "<Btn1Down>:	select_action()";
   
/* Definition der Aktionen fr die Translation-Tabelle */

static XtActionsRec actions[] = {

   {"select_action", (XtActionProc)select_action},
};


/* Klasseninitialisierung fr die XwHistbarClass */

XwHistbarClassRec xwHistbarClassRec = {
  {
     /* core_Class Felder */
     /* Superklasse		*/ (WidgetClass) & xmPrimitiveClassRec,
     /* Klassenname		*/ "XwHistbar",
     /* Widget groesse	    	*/ sizeof(XwHistbarRec),
     /* Class_initialize	*/ NULL,
     /* Class_part_initialize	*/ NULL,
     /* class_inited intern	*/ FALSE,
     /* initialize		*/ Initialize,
     /* initialize_hook (dynami)*/ NULL,
     /* realize	(XCreateWindow)	*/ XtInheritRealize,
     /* actions		    	*/actions,
     /* Anzahl der Action	*/ XtNumber(actions), 
     /* Resourcen 		*/ resources,
     /* Resourcen anzahl	*/ XtNumber(resources),
     /* xrm_class intern	*/ NULLQUARK,
     /* compressmotion 	    	*/ TRUE,
     /* compress_exposure	*/ XtExposeCompressMultiple,
     /* compress_enterleave	*/ TRUE,
     /* VisibilityNotify	*/ FALSE,
     /* destroy		    	*/ Destroy,
     /* resize		    	*/ Resize,	
     /* expose		    	*/ Redisplay,
     /* set_values		*/ SetValues,
     /* set_values_hook	    	*/ NULL,
     /* set_values_almost	*/ XtInheritSetValuesAlmost,
     /* get_values_hook	    	*/ NULL,
     /* accept_focus		*/ NULL,
     /* version		    	*/ XtVersion,
     /* callback_private	*/ NULL,
     /* tm_table		*/defaultTranslations,
     /* query_geometry	    	*/QueryGeometry,
     /* display_accelerator	*/ XtInheritDisplayAccelerator,
     /* extension		*/ NULL
  },
  { /* Primitive_Class felder */
    /* border_highlight	    	*/ NULL,
    /* border_unhighlight	*/ NULL,
    /* translations		*/ XtInheritTranslations,
    /* arm_and_activate	    	*/ NULL,
    /* syn resources		*/ NULL,
    /* num_syn_resources	*/ 0,
    /* extension	 	*/ NULL,
  },
  { /* Whistbar_Class felder */
    /* extensions 		*/ NULL,
  },
};

WidgetClass XwHistbarWidgetClass = (WidgetClass) &xwHistbarClassRec;


static   float  defaultscal[4]   = {0.0,100.0,0.0,100.0};
static   float  defaultgrid[4]   = {0.0,10.0,0.0,10.0};

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	Initialize
 *
 *
 *  Prototype:	static void Initialize (Widget treq, Widget tnew, ArgList args,
 *			 Cardinal *num_args);
 *  
 *  Input:		Widget treq, Widget tnew, ArgList args,
 *			Cardinal *num_args
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	Method, called from X, creating the Widget.
 *			The Instances are checked, dependent parameters
 *			calculated from resources, pixmaps and GC's created
 *			initial clearing of pixmaps, and setting the grid
 *
 *
 *  History:		Sun Jan 18 08:57:43 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
 static void Initialize (Widget treq, Widget tnew, ArgList args, Cardinal *num_args)
{
   XwHistbarWidget new = (XwHistbarWidget) tnew; /* Convert pointer type to Instancerecord */
  float *interm;
  Dimension width,height;

	new->primitive.highlight_thickness = 0;
	/* copy scaling and grid definition over to internal storage, as application might free the array */
	interm = (float *)XtMalloc(sizeof(float)*4);
	if(!interm  ) 
		XtWarning("WHistbar   create   cannot XtMalloc");
	if(new->histbar.vis)
		memcpy(interm, new->histbar.vis , 4*sizeof(float));
	else
		memcpy(interm,  defaultscal , 4*sizeof(float));
	new->histbar.vis = interm;
	interm = (float *)XtMalloc(sizeof(float)*4);
	if(!interm  ) 
		XtWarning("WHistbar   create   cannot XtMalloc");
	if(new->histbar.gr	)
		memcpy(interm, new->histbar.gr , 4*sizeof(float));
	else
		memcpy(interm, defaultgrid  , 4*sizeof(float));
	new->histbar.angle = (M_PI * new->histbar.Angle) / 180.0;	
	new->histbar.gr = interm;	
	new->histbar.sc.ax = new->histbar.gr[0];
	new->histbar.exposed = 0;		/* count to get first exposure */
	new->histbar.lastanncnt = 1;
/*  create default storage for bars */
	new->histbar.bars = (ScBars *)XtMalloc(new->histbar.nbars * sizeof(ScBars));
	  if(!new->histbar.bars ) 
		XtWarning("WHistbar   create   cannot XtMalloc for bars");
	else
		memset(new->histbar.bars , 0 , new->histbar.nbars * sizeof(ScBars));
	
	
  /* Wenn keine Fontliste durch Resourcen gesetzt ist,laden die Default-Fontliste */
  	if (new->histbar.font_list == NULL){
    		new->histbar.font_list =
    		      _XmGetDefaultFontList((Widget) new, XmLABEL_FONTLIST);
  	}
/* set all colours , is not explicitely set, inherit from parent */
	if (new->histbar.background == 0){
	        new->histbar.background = new->core.background_pixel;
	 }

	if(!new->histbar.anncolor)
		new->histbar.anncolor = new->primitive.foreground;
	
  /* Die Fonts fr die Fontliste laden */
  	if (new->histbar.font_list ){
   		 new->histbar.font_list = XmFontListCopy(new->histbar.font_list);
    		_XmFontListGetDefaultFont( new->histbar.font_list, &new->histbar.font_struct);
 	 }
 	 else{
    		new->histbar.font_struct = XLoadQueryFont(XtDisplay(new), "fixed");
 	 }
 	 new->histbar.panno = (Pixmap)NULL;
	get_proportion(new);		/* calculate sizes and starting */

	new->histbar.pix = XCreatePixmap(XtDisplay(new),
  		RootWindow(XtDisplay(new), DefaultScreen(XtDisplay(new))),
  		new->histbar.pixwidth, new->histbar.pixheight,
  		new->core.depth);
	
  	if(new->histbar.anflag ){
	      height = new->histbar.pixheight;
 	      width = new->histbar.annsiz;
  	      if(width > 0 && height > 0) 
  	 
    	        new->histbar.panno = XCreatePixmap(XtDisplay(new),
  		  RootWindow(XtDisplay(new), DefaultScreen(XtDisplay(new))),
  		  width, height,new->core.depth);
  	       else{
  	       	new->histbar.anflag  =  XwHistbarNoAnno;
  	       	XtWarning("Bad size, Annotation set to XwHistbarNoAnno");
  	       }

	 }
	 if(!new->histbar.historysize)       /* use default abscissa size size for history */
	 	new->histbar.historysize  =  XwHistbarDefaultHistsize;
	new->histbar.nactc	 = 0;  	/* preset number of active bars to NO */
	if(new->histbar.firstbar){	/* if set init ID 0 here */
		new->histbar.bars[new->histbar.nactc].color = new->histbar.firstbar;
		new->histbar.bars[new->histbar.nactc].id = new->histbar.nactc;
		new->histbar.bars[new->histbar.nactc].Xpos = 1;
		Gethistbarcolours(new , &(new->histbar.bars[new->histbar.nactc]));	
		new->histbar.nactc++;
		if(new->histbar.history)
			create_history (new);
	}
	 
    	 Gethistbar_gc(new);	
	 scale(new);	/* scale the widget */
	drawgrid(new );

}

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		SetValues
 *  
 *  Input:		Widget curr,Widget requ, Widget new, ArgList args, Cardinal *numargs	
 *
 *  Return:			
 *
 *  Documentation:	Die Funktion wird von Xt als Methode aufgerufen wenn die Applikation 
 * 		  XtSetValue() oder XtSetValues() aufruft. In dieser Funktion wird ber-
 *		  prft ob sich eine Wert der Ressourcen Variablen fr die Widgetklasse
 * 		  gendert hat. Ist dies der Fall werden die davon abhngigen privaten 
  *		  Instanzvariablen neu berechnet.  Wenn sich die Skalierung, der Grid oder
  *		Annotation, bzw. Histbar-Richtung geaendert hat, wird das Widget komplett
  *		neu gezeichnet.
 *	
 *
 *
 *  History:	wizant   Sun Jan 18 08:58:14 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
/* ARGSUSED */

static Boolean SetValues ( Widget curr, Widget requ, Widget new, ArgList args, Cardinal *numargs){

  XwHistbarWidget current = (XwHistbarWidget) curr;
  XwHistbarWidget newon   = (XwHistbarWidget) new;

  int i;
  Boolean rescale = FALSE , reset = FALSE;
  XGCValues values;
  XtGCMask valuemask =  GCFont ;
  float *interm , *oldinterm;
  Dimension width,height;
  
	if(!current->histbar.bars || !current->histbar.gc){	/* indication for bad widget */
		XtWarning("BAD widget id in XwHistbar SetValues");
		return FALSE;
	}

	if(newon->histbar.yndecimals != current->histbar.yndecimals){
		 get_proportion(newon);		/* recalculate sizes and starting */
		rescale = TRUE;
	}
  /* Prfen ob sich die Fontliste gendert hat */
  	if(newon->histbar.font_list != current->histbar.font_list){
    		if((current->histbar.font_list == NULL) && 
    		     (current->histbar.font_struct != NULL))
      			XFreeFont(XtDisplay(current), current->histbar.font_struct);
   		
    		if( current->histbar.font_list)
      			XmFontListFree(current->histbar.font_list);

    		if( newon->histbar.font_list == NULL)
     			newon->histbar.font_list = 
           			      _XmGetDefaultFontList((Widget) newon, XmLABEL_FONTLIST);
  	 	
   	 	if( newon->histbar.font_list != NULL){
     		       newon->histbar.font_list = XmFontListCopy(newon->histbar.font_list);
     		      _XmFontListGetDefaultFont(newon->histbar.font_list,
       			&newon->histbar.font_struct);
    		}
    		else
      		     newon->histbar.font_struct = XLoadQueryFont(XtDisplay(newon),"fixed");
/* now push font to GC's */      		     
  		values.font = newon->histbar.font_struct->fid;
		 XChangeGC(XtDisplay(newon), newon->histbar.gc, valuemask, &values);
		 XChangeGC(XtDisplay(newon), newon->histbar.anngc, valuemask, &values);

   	}	
	if (newon->histbar.barwidth != current->histbar.barwidth)
	    	rescale = TRUE;

/* check and recalculate display relative parameters, recreate pixmaps */
	if( newon->histbar.anflag   != current->histbar.anflag 
	   || newon->histbar.addvalues != current->histbar.addvalues
	   || newon->histbar.barnames   != current->histbar.barnames){
	      if(newon->histbar.pix)
	  	  XFreePixmap(XtDisplay(newon),newon->histbar.pix);
	      if(newon->histbar.panno)
	  	  XFreePixmap(XtDisplay(newon),newon->histbar.panno);
	      newon->histbar.panno = (Pixmap)NULL;	/* put NULL in, there is no pixmap */	
	      newon->histbar.angle = (M_PI * newon->histbar.Angle) / 180.0;	
 
	      get_proportion(newon);		/* recalculate sizes and starting */
	      
 
	      newon->histbar.pix = XCreatePixmap(XtDisplay(newon),
  	                RootWindow(XtDisplay(newon), DefaultScreen(XtDisplay(newon))),
  		newon->histbar.pixwidth, newon->histbar.pixheight,
  		newon->core.depth);
  	      if(newon->histbar.anflag ){
   	                height = newon->histbar.pixheight;
 	      	width = newon->histbar.annsiz;
 	         }
 	     if(width > 0 && height > 0) 
    	        newon->histbar.panno = XCreatePixmap(XtDisplay(newon),
  		  RootWindow(XtDisplay(newon), DefaultScreen(XtDisplay(newon))),
  		  width, height,newon->core.depth);
	   

	 rescale = TRUE;
	}	 
	if(newon->histbar.history != current->histbar.history && newon->histbar.history)
		create_history (newon);
	  	   
/* set all colors if changed or not into GC */

	XSetForeground(XtDisplay(newon) , newon->histbar.gc,newon->primitive.foreground);
	XSetForeground(XtDisplay(newon) , newon->histbar.anngc,newon->histbar.anncolor);
	XSetBackground(XtDisplay(newon) , newon->histbar.gc,newon->histbar.background);
	XSetBackground(XtDisplay(newon) , newon->histbar.anngc,newon->core.background_pixel);
		
	/* copy scaling and grid definition over to internal storage, as application might free the array */
	if(newon->histbar.vis != current->histbar.vis ){
    	   reset = TRUE;
	   for (i=0 ; i< newon->histbar.nactc ; i++){
	      if(newon->histbar.history){
		   newon->histbar.bars[i].history.firstindex = -1;
		   newon->histbar.bars[i].history.lastindex = 0;
		   memset (newon->histbar.bars[i].history.tim,0,sizeof(float)
		   	 * newon->histbar.bars[i].history.size);
		   memset (newon->histbar.bars[i].history.y,0,sizeof(float)
		   	 * newon->histbar.bars[i].history.size);
	       }	 
	       newon->histbar.bars[i].init = 0;	 
	    }
	    oldinterm = current->histbar.vis;
	    interm = 	newon->histbar.vis;
	    newon->histbar.vis = (float *)XtMalloc(sizeof(float)*4);	
  	    if(!newon->histbar.vis ) {
 			 XtWarning("WHistbar   SetValues   cannot XtMalloc");
 			 newon->histbar.vis = oldinterm;
 	    }
 	    else{
			memcpy(newon->histbar.vis , interm , 4*sizeof(float));
			if(current->histbar.vis)
				XtFree((char *)current->histbar.vis);
	    }
	    rescale = TRUE ;
	}
	if(newon->histbar.gr != current->histbar.gr ){
    	   reset = TRUE;
		interm = newon->histbar.gr;	
		oldinterm = current->histbar.gr;
		newon->histbar.gr = (float *)XtMalloc(sizeof(float)*4);	
  		if(!newon->histbar.gr ) {
 			 XtWarning("WHistbar   SetValues   cannot XtMalloc");
 			 newon->histbar.gr = oldinterm;
 		}
 		else{
			memcpy(newon->histbar.gr, interm , 4*sizeof(float));
			if(current->histbar.gr)
				XtFree((char *)current->histbar.gr);
		}
		rescale = TRUE ;
		newon->histbar.sc.ax = newon->histbar.gr[0];	
	}
/* check if number of bars to be allocated has increased */
	if(newon->histbar.nbars > current->histbar.nbars){
		newon->histbar.bars = (ScBars *)XtRealloc(
			(char *)newon->histbar.bars , newon->histbar.nbars * sizeof(ScBars));
		if(!newon->histbar.bars  ) 
 	 		XtWarning("WHistbar   SetValues   cannot XtRealloc");

	}

   	if(newon->histbar.xndecimals != current->histbar.xndecimals)
   		current->histbar.xndecimals =newon->histbar.xndecimals;
	 
	/** now redisplay etc */
	if(rescale){
/* rescale all and clear pixmap, create grid and copy it into visible */
		scale (newon);		/* new scaling */
		clear_histbar(newon);

		if(current->histbar.gr[0] != newon->histbar.gr[0])
	/******************* re-offset */
	    	
    	    	    if(reset){
    	newon->histbar.Xoffset = newon->histbar.pixwidth - newon->histbar.width;
	newon->histbar.Yoffset = 0;
    }
		for(i=0;i<newon->histbar.nactc;i++){		/* first rescale all oldpoints in bars */
    	    	    if(reset){
			newon->histbar.bars[i].init = 0;
		    	newon->histbar.bars[i].oldx =  newon->histbar.pixwidth - newon->histbar.width;
		    	newon->histbar.bars[i].oldy = newon->histbar.bars[i].foldy = 0;
		    	newon->histbar.bars[i].foldx = newon->histbar.bars[i].oldx ;
			if(newon->histbar.history){
		    	    newon->histbar.bars[i].history.firstindex = -1;
		    	    newon->histbar.bars[i].history.lastindex = 0;
			}
		    }
		    else {
    			newon->histbar.bars[i].foldx = (newon->histbar.pixwidth - newon->histbar.width) -
				(newon->histbar.bars[i].lasttim- newon->histbar.sc.ax ) *
	 			newon->histbar.sc.bx * cos(newon->histbar.angle);
	 		newon->histbar.bars[i].oldx = 	newon->histbar.bars[i].foldx;
 			newon->histbar.bars[i].foldy = /* newon->histbar.height + */ 
 			 (newon->histbar.bars[i].lasttim - newon->histbar.sc.ax ) *
	 			newon->histbar.sc.bx * sin(newon->histbar.angle);
	 		newon->histbar.bars[i].oldy = newon->histbar.bars[i].foldy;
		    }
		}
		
	/* need a clear */
		drawgrid(newon);
	 	if(newon->histbar.exposed){
 	     		XClearArea(XtDisplay(newon), XtWindow(newon), newon->histbar.xv, 0, 
 	     		    newon->histbar.width, XtHeight(newon), FALSE);		
			drawannot(newon);
		}
	}
	return TRUE;
}




/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	get_proportion
 *
 *
 *  Prototype:	static void get_proportion(XwHistbarWidget cw);
 *  
 *  Input:		XwHistbarWidget cw
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	calculates all sizedependent parameters inside the
 *			widget. Visible part size and position, pixmap sizes 
 *
 *
 *  History:	Sun Jan 18 08:58:33 CET 1998	
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void get_proportion(XwHistbarWidget cw){

  int  len  , cheight =  (cw->histbar.font_struct->ascent - cw->histbar.font_struct->descent);
  char  annot[40];

	
   	cw->histbar.width = XtWidth(cw)  ;
	cw->histbar.height = XtHeight(cw) ;
	
	cw->histbar.xv = cw->histbar.yv = 0;
		/* correct illegal annotation-flag combinations */
	if(cw->histbar.addvalues)
	    cw->histbar.yv += cheight+2;
	if(cw->histbar.barnames)
	    cw->histbar.yv += cheight+2;
	cw->histbar.height -= cw->histbar.yv  ;
	
	if(cw->histbar.anflag ){
		   sprintf(annot,"% .*f", cw->histbar.yndecimals,(cw->histbar.vis[2]
		          +cw->histbar.vis[3])*10.);
		   len = XTextWidth(cw->histbar.font_struct,annot,strlen(annot)) +8; /* grid space 6 pixels */
		   cw->histbar.width -= len;
		         cw->histbar.xv = len ;
		   sprintf(annot,"% .*f", cw->histbar.xndecimals,cw->histbar.vis[1]*100.);
		   len = XTextWidth(cw->histbar.font_struct,annot,strlen(annot)) +8;
		    cw->histbar.width -= len;	    
		   cw->histbar.annsiz =  len;         /* width of annotation pixmap */    	         
	}
	cw->histbar.pixwidth  = cw->histbar.width * 3;   /* 3 times bigger pixmap */
	cw->histbar.pixheight  = cw->histbar.height * 3;
    	cw->histbar.Xoffset = cw->histbar.pixwidth - cw->histbar.width;
	cw->histbar.Yoffset = 0;

}

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	Gethistbar_gc
 *
 *
 *  Prototype:	static void Gethistbar_gc(XwHistbarWidget  cw);
 *  
 *  Input:		XwHistbarWidget  cw
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	Creates all needed GC's setting the font to all of them
 *			setting background/foreground colours from resources
 *
 *
 *  History:		Sun Jan 18 08:58:51 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void Gethistbar_gc(XwHistbarWidget  cw){

  XGCValues values;
  XtGCMask mask = GCForeground | GCBackground | GCFont ;

  /* Erzeugendes diagram GCs  */
  	values.foreground = cw->primitive.foreground;
  	values.background = cw->histbar.background;
  	values.font = cw->histbar.font_struct->fid;
  	cw->histbar.gc = XCreateGC(XtDisplay(cw),cw->histbar.pix, mask, &values);
  /* Erzeugen des annotation GCs  */
  	values.foreground = cw->histbar.anncolor;
    	values.background = cw->core.background_pixel;
  	cw->histbar.anngc = XCreateGC(XtDisplay(cw),cw->histbar.pix, mask, &values);

/*   Create the GC for dancevals etc */
   	values.foreground = cw->primitive.foreground;
  	values.background = cw->histbar.background;
  	values.font = cw->histbar.font_struct->fid;
 	
	clear_histbar(cw );
 

 
}


/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	Redisplay
 *
 *
 *  Prototype:	static void Redisplay(Widget w, XEvent *event, Region region);
 *  
 *  Input:		Widget w, XEvent *event, Region region
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	This is the Expose method for X
 *		The region and event parameter are ignored.
 *		actually the active pixmap parts are replotted and the
 *		annotation redrawed
 *
 *
 *  History:	Sun Jan 18 08:59:13 CET 1998	
 *
 *
 ***************************************************************************
 ***************************************************************************/

static void Redisplay(Widget w, XEvent *event, Region region){

  XwHistbarWidget cw = (XwHistbarWidget) w;
  XwHistbarCallbackStruct call_value;

  /* Prfen ob die Widgetinstanz schon realisiert ist */
 	 if ( !XtIsRealized((Widget)cw) )
		return;
	
  /* Fenster aktualisieren */  
  	copypix(cw , 1);
 	 cw->histbar.exposed++;
 	 drawannot(cw);
 	 if(cw->histbar.expose_callback){
   		 call_value.reason = XmCR_EXPOSE;
    		call_value.event = event;
   		 call_value.fd = -1;
   		 call_value.width = cw->histbar.width;
   		 call_value.height = cw->histbar.height;
/*********   		 call_value.value = cw->histbar.lastpush;    !!!!!!!!!!!!!!!!!!*/
    /* Aufrufen der Callbackliste fr den Exposeevent */
   		 XtCallCallbackList((Widget)cw,cw->histbar.expose_callback,&call_value);
 	 }
}



/* ======================================================================= */
/* 
   Funktionsname: QueryGeometry 
   Input:	  Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer
   Beschreibung:  
*/
/* ARGSUSED */

static XtGeometryResult QueryGeometry( Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer){

   	XtWarning("XwHistbar: query Geometry.");
  	answer->request_mode = (CWWidth | CWHeight);
      	answer->width = proposed->width;

  	return XtGeometryYes;
}




/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	Resize
 *
 *
 *  Prototype:	static void Resize ( Widget w );
 *  
 *  Input:		Widget w
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	This is a method, called by X whenever the size
 *			of widget was changed. it recreates the pixmaps for
 *			the new size. 
 *
 *
 *  History:		Sun Jan 18 08:51:44 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/

static void Resize ( Widget w ){

  XwHistbarWidget cw = (XwHistbarWidget) w;
  Dimension width,height;
  int i , idx , ID ;
  XwHistbarCallbackStruct call_value;

	XFreePixmap(XtDisplay(cw), cw->histbar.pix);
	if(cw->histbar.panno)
    		XFreePixmap(XtDisplay(cw),cw->histbar.panno);

  	 cw->histbar.panno = (Pixmap)NULL;	/* preclear */
   	 get_proportion( cw );		/* recalculate sizes and starting */
    	cw->histbar.pix =XCreatePixmap(XtDisplay(cw),
  		RootWindow(XtDisplay(cw), DefaultScreen(XtDisplay(cw))),
  		cw->histbar.pixwidth, cw->histbar.pixheight,
  		cw->core.depth);
/* pixmap for external annotation  */ 
  	if(cw->histbar.anflag ){
		height =  cw->histbar.pixheight;
 	      	width = cw->histbar.annsiz;
 	    	 if(width > 0 && height > 0)
    	       		 cw->histbar.panno = XCreatePixmap(XtDisplay(cw),
  			       RootWindow(XtDisplay(cw), DefaultScreen(XtDisplay(cw))),
  			       width, height,cw->core.depth);
	 }
	clear_histbar(cw );
	scale(cw);
 /* re-offset      all lastpoints*/
 	
	for(i=0;i<cw->histbar.nactc;i++){		/* first rescale all oldpoints in bars */
		cw->histbar.bars[i].Xpos =
		       i * cw->histbar.barfactor * cw->histbar.barwidth + cw->histbar.barwidth;
		cw->histbar.bars[i].init = 0;
		cw->histbar.bars[i].oldx =  cw->histbar.pixwidth - cw->histbar.width;
		cw->histbar.bars[i].oldy = cw->histbar.bars[i].foldy = 0;
		cw->histbar.bars[i].foldx = cw->histbar.bars[i].oldx ;
	}
	cw->histbar.gr[0] = cw->histbar.prevgrid;
	
	drawgrid(cw );		/* redraw last grid */
	if(cw->histbar.exposed)
	     XClearArea(XtDisplay(cw), XtWindow(cw), 
	        cw->histbar.xv, 0, cw->histbar.width, XtHeight(cw), FALSE);		

         if(cw->histbar.history){		/* existing histbar history, so redraw all */
            for(ID = 0 ; ID < cw->histbar.nactc ; ID++){
           		
  		idx = cw->histbar.bars[ID].history.firstindex;
 		if(idx<0)
 		    idx = 1;
 		if(idx == cw->histbar.bars[ID].history.lastindex 
 	  	 || (cw->histbar.bars[ID].history.firstindex < 0
 	  	 	 && !cw->histbar.bars[ID].history.lastindex ))
 			continue;
 			
		cw->histbar.bars[ID].lasttim = cw->histbar.bars[ID].history.tim[idx];
		cw->histbar.bars[ID].lasty = cw->histbar.bars[ID].history.y[idx];
 		while(++idx != cw->histbar.bars[ID].history.lastindex){
 	      		if(idx >=  cw->histbar.bars[ID].history.size)
 		    	     idx = 0; 
 		    	if(cw->histbar.bars[ID].history.tim[idx] <  cw->histbar.sc.ax)
 		    		continue;   
			drawbar	(cw ,  ID ,   cw->histbar.bars[ID].history.y[idx]
				, cw->histbar.bars[ID].history.tim[idx]); 
			cw->histbar.bars[ID].lasttim = cw->histbar.bars[ID].history.tim[idx];
			cw->histbar.bars[ID].lasty = 	cw->histbar.bars[ID].history.y[idx];  
 		}
            }
	if(cw->histbar.exposed)
           		  copypix(cw , 1);

          }
	 if(cw->histbar.resize_callback){
   		 call_value.reason =XmCR_RESIZE ;
    		call_value.event =NULL;
   		 call_value.fd = -1;
   		 call_value.width = cw->histbar.width;
   		 call_value.height = cw->histbar.height;
/***********   		 call_value.value = cw->histbar.lastpush;    !!!!!!!!!!!!!!!!*/
    /* Aufrufen der Callbackliste fr den Exposeevent */
   		 XtCallCallbackList((Widget)cw,cw->histbar.resize_callback,&call_value);
 	 }
	
}



/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		Destroy
 *  
 *  Input:			Widget w 
 *
 *  Return:			
 *
 *  Documentation:		Method, called when the widget is destroyed
 *			(not called during termination and quitting
 *
 *
 *  History:		
 *
 ***************************************************************************
 ***************************************************************************/
static void Destroy(Widget w ){
  XwHistbarWidget cw = (XwHistbarWidget) w;
 int ID;
 unsigned long colorpix[2];

  	XFreeGC(XtDisplay( cw), cw->histbar.gc);
    	XFreeGC(XtDisplay( cw), cw->histbar.anngc);
	/* XmFontListFree(cw->histbar.font_list); */
	XFreePixmap(XtDisplay(cw), cw->histbar.pix);
	if(cw->histbar.panno)
		XFreePixmap(XtDisplay(cw), cw->histbar.panno);
		
	XtFree((char *)cw->histbar.vis);
	XtFree((char *)cw->histbar.gr);
	cw->histbar.pix = 0;
	cw->histbar.panno = 0;
	cw->histbar.gc = NULL;
	cw->histbar.anngc = NULL;
	/* cw->histbar.font_list = NULL; */
	 
	 	/* free all history and bars created */
        if(cw->histbar.history && cw->histbar.bars ){	/* existing histbar history, so free all */
            for(ID = 0 ; ID < cw->histbar.nactc ; ID++){
		if(cw->histbar.bars[ID].history.tim)	
		 	XtFree((char *)cw->histbar.bars[ID].history.tim); 
		if(cw->histbar.bars[ID].history.y)		 	
		 	XtFree((char *)cw->histbar.bars[ID].history.y); 
		if(cw->histbar.bars[ID].fixtext)
		    	free(cw->histbar.bars[ID].fixtext);
	 	cw->histbar.bars[ID].history.size =  0;
		cw->histbar.bars[ID].history.firstindex = -1;
	 	cw->histbar.bars[ID].history.lastindex = 0;
	    	colorpix[0] = cw->histbar.bars[ID].color;
	    	colorpix[1] = cw->histbar.bars[ID].shadecolor;	
		XFreeColors(XtDisplay(cw), DefaultColormap(XtDisplay(cw)
		,DefaultScreen(XtDisplay(cw))),colorpix,2,(int)NULL);		

	    }
	}
	if(cw->histbar.bars )
		XtFree((char *)cw->histbar.bars);
	cw->histbar.bars = NULL;
	 
 
}
/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	clear_histbar
 *
 *
 *  Prototype:	static void clear_histbar(XwHistbarWidget cw );
 *  
 *  Input:		XwHistbarWidget cw , int notall
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	 actual clearing the total pixmap 
 *			to background colour, should be followed by recreating grid 
 *			(and scale)
 *
 *
 *  History:		Sun Jan 18 08:52:05 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void clear_histbar(XwHistbarWidget cw ){


     	XSetForeground(XtDisplay(cw),cw->histbar.gc,cw->histbar.background);
     	XSetForeground(XtDisplay(cw),cw->histbar.anngc,cw->core.background_pixel);
	XFillRectangle(XtDisplay(cw),cw->histbar.pix,cw->histbar.gc,
		    0,0,cw->histbar.pixwidth,cw->histbar.pixheight);
     
	if(cw->histbar.panno)
 	            XFillRectangle(XtDisplay(cw),cw->histbar.panno,cw->histbar.anngc,
		    0,0,cw->histbar.annsiz,cw->histbar.pixheight);
	
      	XSetForeground(XtDisplay(cw),cw->histbar.anngc,cw->histbar.anncolor);

}



/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	scale
 *
 *
 *  Prototype:	static void scale(XwHistbarWidget cw);
 *  
 *  Input:		XwHistbarWidget cw
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	does all scaling,construct parameters to calculate from users values 
 *			into pixmap. *
 *		given vis[4]	vis[0]   xoffset starting value of time
 *				vis[1]   displayable x-range 
 *				vis[2]   lowest y value
 *				vis[3]   range of y
 *  History:	Sun Jan 18 08:52:39 CET 1998
	
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void scale(XwHistbarWidget cw){
   double width,height;
 /* assumed, that pixel coords begin with 0 */	
 
 
 	width  = (cw->histbar.height / 2.0) ;/* / sin(cw->histbar.angle);*/
 	
 	height = cw->histbar.height /2.0 ;
 	
	cw->histbar.sc.bx =  width /  cw->histbar.vis[1] ;
	
	
	cw->histbar.sc.by = height /  cw->histbar.vis[3] ;
	cw->histbar.sc.ay =  - cw->histbar.sc.by * cw->histbar.vis[2];
}



/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	drawannot
 *
 *
 *  Prototype:	static void drawannot(XwHistbarWidget  cw);
 *  
 *  Input:		XwHistbarWidget  cw
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:	draws the ordinate annotation(if needed) to window
 *
 *
 *  History:	Mon Jan  5 10:15:22 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void drawannot(XwHistbarWidget  cw){
   int ypix  , cnt = cw->histbar. ynthgridanno+1
   , len,cheight =  (cw->histbar.font_struct->ascent - cw->histbar.font_struct->descent);
   int  pos=0 , i;
   float base = cw->histbar.gr[2]  ;
  char annot[40];
  
 
      	if(!cw->histbar.anflag)
      			return;
       	do {   	/* ordinate annot */
  		      ypix = cw->histbar.height -  (cw->histbar.sc.ay + cw->histbar.sc.by * base);
  		      
      		     if(ypix <= cw->histbar.height  ){
       		          if(!--cnt ){
       			     cnt = cw->histbar.ynthgridanno+1;
       			     sprintf(annot,"%.*f",cw->histbar.yndecimals,base);
       			     len = XTextWidth(cw->histbar.font_struct,annot,strlen(annot)) ;
     			        pos = cw->histbar.xv - len -8	;
       			     XDrawString (XtDisplay(cw),XtWindow(cw),cw->histbar.anngc
       			        ,pos , ypix+cheight/2 ,annot,strlen(annot));			      
      			  }
      		     }
      		     XDrawLine(XtDisplay(cw),XtWindow(cw),cw->histbar.anngc
      		         , cw->histbar.xv , ypix,cw->histbar.xv -8 ,ypix);
      		     base += cw->histbar.gr[3];
      	}while(ypix >=cw->histbar.height/2  );
     	if(!cw->histbar.barnames)
	    return;
	    
	    	    /* now add the fixtext per bar  */
	ypix = (cw->histbar.height + 2 ) 	    
	    +( (cw->histbar.addvalues) ? 2 * cheight : cheight );
	for( i= 0 ; i < cw->histbar.nactc ; i++){
	    if(	cw->histbar.bars[i].fixtext){
	    	XSetForeground(XtDisplay(cw),cw->histbar.anngc
		    ,cw->histbar.bars[i].color);
	  	pos = cw->histbar.xv + cw->histbar.bars[i].Xpos - cw->histbar.barwidth;
  		if((pos + cw->histbar.barwidth) < (cw->histbar.xv + cw->histbar.width))  	    
		   XDrawString (XtDisplay(cw),XtWindow(cw),cw->histbar.anngc
       		    	,pos , ypix  ,cw->histbar.bars[i].fixtext
		    	,strlen(cw->histbar.bars[i].fixtext));
	    }
	}
	XSetForeground(XtDisplay(cw),cw->histbar.anngc,cw->histbar.anncolor);
  }


/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:	drawgrid
 *
 *
 *  Prototype:	static void drawgrid(XwHistbarWidget cw );
 *  
 *  Input:		XwHistbarWidget cw , int doit
 *
 *  Output:		
 *
 *  Return:		
 *
 *  Documentation:		
 *			Draws the grid into pixmap and draws the time
 *			annotation  into annotation pixmap
 *
 *
 *  History:		Mon Jan  5 11:19:36 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void drawgrid(XwHistbarWidget cw){
   int ypix , cnt = cw->histbar.lastanncnt , len;
   float base = cw->histbar.gr[0] ;
   char annot[40];
   
   int cheight =  (cw->histbar.font_struct->ascent - cw->histbar.font_struct->descent);

    	if(!cw->histbar.panno)
	    return;
/*    grid forground colour is fixed set to cw->primitive.foreground    */
      	XSetForeground(XtDisplay(cw),cw->histbar.anngc,cw->core.background_pixel);
  	XFillRectangle(XtDisplay(cw),cw->histbar.panno,cw->histbar.anngc,
		    0,0,cw->histbar.annsiz,cw->histbar.pixheight);
      	XSetForeground(XtDisplay(cw),cw->histbar.anngc,cw->histbar.anncolor);
/* do y- rastering */      	
	cw->histbar.prevgrid = cw->histbar.gr[0];   /* save for repeat gridding */
      	do {   	/* time grid first up*/
       	
 			ypix = cw->histbar.height + 
 			 (base - cw->histbar.sc.ax ) *
	 			cw->histbar.sc.bx * sin(cw->histbar.angle);
      			if(ypix >= 0){
      			     if(!--cnt ){
       			          cnt = cw->histbar.xnthgridanno+1;
       			          sprintf(annot,"%.*f",cw->histbar.xndecimals,base);
       			          len = XTextWidth(cw->histbar.font_struct,annot,strlen(annot)) ;
     			          
       			          XDrawString (XtDisplay(cw),cw->histbar.panno,cw->histbar.anngc
       			        ,cw->histbar.annsiz - len  , ypix+cheight/2 ,annot,strlen(annot));			      
			     }
      			    XDrawLine(XtDisplay(cw),cw->histbar.panno,cw->histbar.anngc
      				 ,0,ypix,8,ypix);
     			}
      			base -= cw->histbar.gr[1];
      	}while(ypix >= 0);
   	base = cw->histbar.gr[0] ;
          	do {   	/* time grid second down*/
 			
			ypix = cw->histbar.height +
			      (base - cw->histbar.sc.ax ) *
	 			cw->histbar.sc.bx * sin(cw->histbar.angle);
      			if( ypix <= cw->histbar.pixheight && ypix >= 0){
     			     if(!--cnt ){
       			          cnt = cw->histbar.xnthgridanno+1;
       			          sprintf(annot,"%.*f",cw->histbar.xndecimals,base);
       			          len = XTextWidth(cw->histbar.font_struct,annot,strlen(annot)) ;
     			          
       			          XDrawString (XtDisplay(cw),cw->histbar.panno,cw->histbar.anngc
       			            ,cw->histbar.annsiz - len , ypix+cheight/2
       			             ,annot,strlen(annot));
       			           			      
			     }      			
      			     XDrawLine(XtDisplay(cw),cw->histbar.panno,cw->histbar.anngc
      				 ,0,ypix,8,ypix);
      		 	     cw->histbar.gr[0] = base;  
      			}
      			base += cw->histbar.gr[1];
      	}while(ypix <= cw->histbar.pixheight);
   	cw->histbar.lastanncnt = cnt;
   	
}

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		Gethistbarcolours
 *  
 *  Input:			XwHistbarWidget cw
 *			 ScBars  *bars
 *
 *  Output:		bars->shadecolor
 *
 *  Return:			
 *
 *  Documentation:		Queries the given colour to get the RGB values,
 *			darkens the RGB by 0.7 and allocates the darker colour
 *			to shadecolour.
 *
 *   On host:		wizant			by: WizAnt Software
 *  History:		Mon Dec 29 14:34:55 CET 1997
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void Gethistbarcolours	 (XwHistbarWidget cw , ScBars  *bars){

  XColor hardware ;
  
  	hardware.pixel = bars->color;
  	XQueryColor(XtDisplay(cw),DefaultColormap(XtDisplay(cw),DefaultScreen(XtDisplay(cw)))
  			,&hardware);
	hardware.red = (unsigned short)((float)hardware.red  * 0.7);
	hardware.green = (unsigned short)((float)hardware.green  * 0.7);
	hardware.blue = (unsigned short)((float)hardware.blue  * 0.7);
	XAllocColor(XtDisplay(cw),DefaultColormap(XtDisplay(cw),DefaultScreen(XtDisplay(cw)))
		,&hardware);
	bars->shadecolor = hardware.pixel;
}



/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		 copyannot
 *  
 *  Input:			XwHistbarWidget  cw , Drawable target,double yoffset
 *
 *  Return:			
 *
 *  Documentation:		if external annotation set, copies the annotation pixmap
 *			in place for annotation, wher given might be x or y coordinate
 *			corresponding to direction
 *
 *
 *  History:		Wed Mar 12 17:45:42 CET 1997
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void  copyannot(XwHistbarWidget  cw , Drawable target , double yoffset){
  
  	if(!cw->histbar.panno)
  		return;		/* no copy needed */
  	XCopyArea(XtDisplay(cw),cw->histbar.panno,target,
  		        cw->histbar.anngc,   0 ,(int)yoffset
  		         , cw->histbar.annsiz, cw->histbar.height
  		          ,cw->histbar.xv+cw->histbar.width,0);
}

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		 copypix
 *  
 *  Input:			XwHistbarWidget  cw
 *  	    	    	    	int copy    	    if zero, don't copy really
 *
 *  Return:			
 *
 *  Documentation:		copies the actual pixmapwindow into visible
 *			if external annotation set, copies the annotation pixmap
 *			in place for annotation.
 *
 *  History:		Sat Jan  3 09:26:31 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void  copypix(XwHistbarWidget  cw , int copy){
  int i;
  double lsttim = 0;
  	for(i = 0 ; i < cw->histbar.nactc ; i++){     /* get the minimum footpoint of bars */
  		if(cw->histbar.bars[i].foldy > 0
  		|| cw->histbar.bars[i].foldx < cw->histbar.pixwidth - cw->histbar.width){
		    if(cw->histbar.bars[i].foldy > cw->histbar.Yoffset)
  			cw->histbar.Yoffset = cw->histbar.bars[i].foldy;
		    if(cw->histbar.bars[i].foldx < cw->histbar.Xoffset)
  			cw->histbar.Xoffset = cw->histbar.bars[i].foldx;
  		    if(lsttim < cw->histbar.bars[i].lasttim)
  				lsttim = cw->histbar.bars[i].lasttim;
  		}
  	}
	if(copy){
	    XCopyArea(XtDisplay(cw),cw->histbar.pix,XtWindow(cw),
  	    	cw->histbar.gc, (int)cw->histbar.Xoffset  , (int)cw->histbar.Yoffset
  	    	, cw->histbar.width, cw->histbar.height,cw->histbar.xv,0);
  	   
	    	copyannot(cw ,XtWindow(cw),cw->histbar.Yoffset );     
	    	dancevals(cw);
	    }
	if(cw->histbar.Xoffset <= 10 
	      || cw->histbar.Yoffset >= (cw->histbar.pixheight -cw->histbar.height -10)){
		/* now refresh pixmap */
		XCopyArea(XtDisplay(cw),cw->histbar.pix,cw->histbar.pix
		  , cw->histbar.gc, (int)cw->histbar.Xoffset  , (int)cw->histbar.Yoffset
		  ,cw->histbar.width, cw->histbar.height
		  ,cw->histbar.pixwidth - cw->histbar.width,0);
		 cw->histbar.Xoffset  =  cw->histbar.pixwidth - cw->histbar.width;
		 cw->histbar.Yoffset = 0;/* cw->histbar.height; */
		for(i = 0 ; i < cw->histbar.nactc ; i++){
		    cw->histbar.bars[i].oldx = cw->histbar.pixwidth - cw->histbar.width;
		    cw->histbar.bars[i].oldy = cw->histbar.bars[i].foldy =  0 ;
		    cw->histbar.bars[i].foldx = cw->histbar.bars[i].oldx ;
		}
		XSetForeground(XtDisplay(cw),cw->histbar.gc,cw->histbar.background);  
  		XFillRectangle(XtDisplay(cw),cw->histbar.pix , cw->histbar.gc, 
		     0,0, cw->histbar.pixwidth - cw->histbar.width , cw->histbar.pixheight);
		XFillRectangle(XtDisplay(cw),cw->histbar.pix , cw->histbar.gc, 
		     cw->histbar.pixwidth - cw->histbar.width ,    cw->histbar.height
		      ,cw->histbar.width, cw->histbar.pixheight -  cw->histbar.height);
		cw->histbar.sc.ax =  lsttim;
		drawgrid(cw); 
	}
	
}




/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		drawbar
 *  
 *  Input:			XwHistbarWidget cw
 *			 long int id		id of actual bar
 *			float val , float time      	values of bar
 *
 *  Output:		
 *
 *  Return:			
 *
 *  Documentation:		draws a bar into pixmap, saving the actual values for next
 *			draw. The deepness of bar is calculated from previous time
 *
 *   On host:		wizant			by: WizAnt Software
 *  History:		Sat Jan  3 09:26:15 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void drawbar (XwHistbarWidget cw,  long int id, float val , float time){

  long int dx,dy;
  XPoint rect[4],top[4],shade[4];

	if(time < cw->histbar.bars[id].lasttim)
			return;		/* x-axis must increase */

	if((time - cw->histbar.bars[id].lasttim	) >  (cw->histbar.vis[0] + cw->histbar.vis[1])){
		fprintf(stderr," time is out of scale: id:%ld, prevtime / time %f %f   %f\n" 
		,id,cw->histbar.bars[id].lasttim,time , cw->histbar.vis[1]);
			
			  /* ignoring is not the best, think about something intelligent */
	}
	
	memset(rect,0,sizeof(rect));		/* preclear plotting arrays */
	memset(shade,0,sizeof(shade));
	memset(top,0,sizeof(top));
	cw->histbar.bars[id].foldx = (cw->histbar.pixwidth - cw->histbar.width) -
		(time - cw->histbar.sc.ax ) *
	 	cw->histbar.sc.bx * cos(cw->histbar.angle);
	cw->histbar.bars[id].foldy =  
 		(time - cw->histbar.sc.ax ) *
		 cw->histbar.sc.bx * sin(cw->histbar.angle);
	
	if(cw->histbar.bars[id].foldx > cw->histbar.pixwidth 
		|| cw->histbar.bars[id].foldx < 0
		||  cw->histbar.height +cw->histbar.bars[id].foldy < 0
		||  cw->histbar.height +cw->histbar.bars[id].foldy > cw->histbar.pixheight){
	}
	dy = (((int)cw->histbar.bars[id].foldy - cw->histbar.bars[id].oldy) < 1)
		? 1 : (int)cw->histbar.bars[id].foldy - cw->histbar.bars[id].oldy;
	dx = (cw->histbar.bars[id].oldx - ((int)cw->histbar.bars[id].foldx) < 1)
		? 1 : cw->histbar.bars[id].oldx - (int)cw->histbar.bars[id].foldx  ;
	cw->histbar.bars[id].oldx = cw->histbar.bars[id].foldx;
	cw->histbar.bars[id].oldy = cw->histbar.bars[id].foldy;
	
	if(!cw->histbar.bars[id].init++)
		dx = dy = 2;
	top[1].y = shade[2].y = (short int)-dy;
	top[3].y                     =  (short int)dy;
	top[1].x = shade[2].x = (short int) dx;
	top[3].x                     = (short int)-dx;
		
	top[0].x = shade[0].x = rect[0].x = cw->histbar.bars[id].Xpos + cw->histbar.bars[id].foldx;
	shade[0].y = rect[0].y = cw->histbar.height + cw->histbar.bars[id].foldy;
	
	rect[2].y =  (cw->histbar.sc.ay + cw->histbar.sc.by * val);
	top[2].x  = rect[1].x = -cw->histbar.barwidth;
 	rect[3].x =                   cw->histbar.barwidth;
	if(rect[2].y < 0)
		rect[2].y = 0;
	shade[3].y = rect[2].y;
	top[0].y = shade[0].y -  rect[2].y;
	rect[2].y = shade[1].y = -rect[2].y;
	
	XSetForeground(XtDisplay(cw), cw->histbar.gc,cw->histbar.bars[id].color);
	XFillPolygon(XtDisplay(cw),cw->histbar.pix , cw->histbar.gc, rect
			, 4, Convex, CoordModePrevious);
	XSetForeground(XtDisplay(cw), cw->histbar.gc,0);
	XDrawLines(XtDisplay(cw),cw->histbar.pix , cw->histbar.gc, top
		,2,CoordModePrevious);
	XFillPolygon(XtDisplay(cw),cw->histbar.pix , cw->histbar.gc, top
			, 4, Convex, CoordModePrevious);
	XSetForeground(XtDisplay(cw), cw->histbar.gc,cw->histbar.bars[id].shadecolor);
	XFillPolygon(XtDisplay(cw),cw->histbar.pix , cw->histbar.gc, shade
			, 4, Convex, CoordModePrevious);
	cw->histbar.bars[id].lasttim = time;
	cw->histbar.bars[id].lasty     = val;	
}		
 	
		
/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		dancevals
 *  
 *  Input:			XwHistbarWidget  cw
 *
 *  Return:			
 *
 *  Documentation:		checks if the flag for display values is set
 *			and draws the last value at inhistbaring position
 *
 *
 *  History:		Sun Jan 25 09:24:39 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void  dancevals(XwHistbarWidget  cw ){
  int i , xpix,ypix ;
  int cheight =  (cw->histbar.font_struct->ascent - cw->histbar.font_struct->descent);
  char annot[20];
  
   	    if(!cw->histbar.addvalues )
	    	    return;
 
	ypix = cw->histbar.height  + cheight ;
    	XClearArea(XtDisplay(cw), XtWindow(cw), cw->histbar.xv,cw->histbar.height , 
 	     	cw->histbar.width, cheight, FALSE);		

	for (i = 0; i < cw->histbar.nactc ; i++){
  	    	sprintf(annot,"%.*f",cw->histbar.yndecimals+1,cw->histbar.bars[i].lasty);
		XSetForeground(XtDisplay(cw),cw->histbar.gc,cw->histbar.bars[i].color);  
  		xpix = cw->histbar.xv + cw->histbar.bars[i].Xpos - cw->histbar.barwidth;
  		if((xpix + cw->histbar.barwidth) < (cw->histbar.xv + cw->histbar.width))  
 	    	   XDrawString (XtDisplay(cw),XtWindow(cw),cw->histbar.gc
       			 ,xpix , ypix ,annot,strlen(annot));		 
       	}
}



/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		add_history
 *  
 *  Input:			XwHistbarWidget  cw
 *			long int id		bar id
 *
 *  Return:			
 *
 *  Documentation:		used from addbar to initialize history
 *
 *
 *  History:		Sun Jan 18 08:53:52 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void  add_history	(XwHistbarWidget  cw, long int id){

	if(!cw->histbar.bars[id].history.tim){	
		    
		 cw->histbar.bars[id].history.tim 
		 	= (float *)XtMalloc(sizeof(float)* cw->histbar.historysize);
		 cw->histbar.bars[id].history.y 
		 	= (float *)XtMalloc(sizeof(float)* cw->histbar.historysize);
		   if(! cw->histbar.bars[id].history.y || !cw->histbar.bars[id].history.tim ) {
 	 		XtWarning("WHistbar  add_history    cannot XtMalloc, history OFF");
			cw->histbar.history = 0;
		}
	 	cw->histbar.bars[id].history.size =  cw->histbar.historysize;
	}
/* even if a history buffer was existing, clear it an reset the pointers */
	if(cw->histbar.history){
	     cw->histbar.bars[id].history.firstindex = -1;
	     cw->histbar.bars[id].history.lastindex = 0;
	     memset (cw->histbar.bars[id].history.tim,0,sizeof(float)* cw->histbar.bars[id].history.size);
	     memset (cw->histbar.bars[id].history.y,0,sizeof(float)* cw->histbar.bars[id].history.size);
	}
}


/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		addhistory
 *  
 *  Input:			Histbarhistory *hist		history structure
 *			 float xy[]		abscissa and ordinate
 * 
 *  Return:			
 *
 *  Documentation:		maintains the history pointers and adds a pair of data
 *
 *
 *  History:		Sun Jan 18 08:54:12 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void  addhistory   (Histbarhistory *hist, float xy[]){

	if(!hist)
		return;	/* no history, ignore */
	if(++hist->lastindex >= hist->size){
		hist->lastindex = 0;
		hist->firstindex = 1 ;
	}
	if(hist->firstindex >= 0 && ++hist->firstindex >= hist->size)
			 hist->firstindex= 0;
	hist->tim[hist->lastindex] = xy[0];
	hist->y[hist->lastindex] = xy[1];
			 
}
/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		create_history
 *  
 *  Input:			XwHistbarWidget  cw
 *
 *  Return:			
 *
 *  Documentation:		Creates ( or recreates) basic history structures
 *			for all allocated bar structures, history buffers
 *			are only created for activated bar structures
 *			if(size of buffer is not the same as given by resource , or if
 *			different from visible diagram pixels size, the buffere is reallocated
 *			old contents is lost;
 *
 *  History:		Sun Jan 18 08:54:35 CET 1998

 *
 *
 ***************************************************************************
 ***************************************************************************/
static void  create_history 		(XwHistbarWidget  cw){
   int i;
   	
   	for (i=0 ; i< cw->histbar.nactc ; i++){
		if(cw->histbar.bars[i].history.tim){	/* might be changed */
		    /* clear old contents to a new allocated array  */
		     free(cw->histbar.bars[i].history.tim);
		     free(cw->histbar.bars[i].history.y);
		 }

		 cw->histbar.bars[i].history.tim 
			 	= (float *)XtMalloc(sizeof(float)* cw->histbar.historysize);
		 cw->histbar.bars[i].history.y 
			 	= (float *)XtMalloc(sizeof(float)* cw->histbar.historysize);
		 if(! cw->histbar.bars[i].history.y || !cw->histbar.bars[i].history.tim ) {
 	 		XtWarning("WHistbar  create_history    cannot XtMalloc, history OFF");
			cw->histbar.history = 0;
		}
		else{
		      cw->histbar.bars[i].history.size =  cw->histbar.historysize;
		      cw->histbar.bars[i].history.firstindex = -1;
		      cw->histbar.bars[i].history.lastindex = 0;
		      memset (cw->histbar.bars[i].history.tim,0,sizeof(float) * cw->histbar.historysize);
		      memset (cw->histbar.bars[i].history.y,0,sizeof(float) * cw->histbar.historysize);
		}
	}	 
}



/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		select_action
 *  
 *  Input:			Widget w, XEvent *event
 *
 *  Return:			
 *
 *  Documentation:	action routine called at button 1 down
 *			calculation actual abscissa value and putting
 *			it into callback structure. calling eventally 
 *			user Input callback
 *
 *
 *  History:		Thu Jan 29 17:55:41 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
static void select_action(Widget w, XEvent *event){
 XwHistbarWidget  cw = (XwHistbarWidget) w ;
 XwHistbarCallbackStruct call_value;
 int i , k , pos;
 

 
  if(cw->histbar.input_callback){
    	call_value.reason = XmCR_INPUT;
    	call_value.event = event;
    	call_value.fd = -1;
   	i = cw->histbar.Yoffset + event->xbutton.y - cw->histbar.height  ; 
 /*    extract from cursor position the time value and the Xpos (bar id)
 */		    			
	call_value.time_win = cw->histbar.sc.ax 
		+ i /(cw->histbar.sc.bx * sin(cw->histbar.angle));
	i =event->xbutton.x -cw->histbar.xv - 
	    (  cw->histbar.height - event->xbutton.y) / tan(cw->histbar.angle);
	if(i < 0 || cw->histbar.nactc <= 1 
	  || i <= (cw->histbar.bars[0].Xpos + cw->histbar.barwidth)){
	    pos = cw->histbar.bars[0].Xpos;
	    call_value.id = 0;	    /* must be the first id */
	}
	else {
	    call_value.id = -1;
	    for(k = 0 ; k < cw->histbar.nactc -1 ; k++){
	    	if(i <= cw->histbar.bars[k+1].Xpos + cw->histbar.barwidth
		     && i >= (cw->histbar.bars[k].Xpos + cw->histbar.barwidth  )){
    	    	     pos = cw->histbar.bars[k+1].Xpos ;
		     call_value.id = cw->histbar.bars[k+1].id;
		     break;
		}
	    }
	}
	if(call_value.id >= 0){
	    i = cw->histbar.Xoffset + event->xbutton.x -cw->histbar.xv - pos ;
	    call_value.time_id = cw->histbar.sc.ax 
	    	- (i - cw->histbar.pixwidth + cw->histbar.width)
	    	/(cw->histbar.sc.bx * cos(cw->histbar.angle));
	}
	else
	    call_value.time_id = 0.0;
   	call_value.width = cw->histbar.width;
   	call_value.height = cw->histbar.height;
     XtCallCallbackList((Widget)cw,cw->histbar.input_callback, &call_value);
    } 

}
/* ================================================================================ */
/*		Public directly accessible Functions */
/* ================================================================================ */


/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		XwCreateHistbar
 *  
 *  Input:			Widget parent,
 *			 char *name,
 *			ArgList arglist,
 *			Cardinal argcount
 *
 *  Return:		 Widget	
 *
 *  Documentation:		Standeard X- creating routine
 *
 *
 *  History:		Sun Jan 18 08:55:04 CET 1998 on host:wizant
 *			
 *
 *
 ***************************************************************************
 ***************************************************************************/
 Widget XwCreateHistbar( 
                        Widget parent,
                        char *name,
                        ArgList arglist,
                        Cardinal argcount) {
                      
 
   return( (Widget) XtCreateWidget(name, XwHistbarWidgetClass, parent, arglist, argcount));

 }



/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		XwHistbarchangecolor
 *  
 *  Input:			Widget w, int ID , Pixel color
 *
 *  Return:			
 *
 *  Documentation:		changes the colour for the selected bar
 *
 *
 *  History:		Sun Jan 18 08:56:03 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
void XwHistbarchangecolor(Widget w, int ID , Pixel color){
  XwHistbarWidget cw = (XwHistbarWidget) w;
  unsigned long colorpix[2];

	if(!cw->histbar.bars || !cw->histbar.gc){	/* indication for bad widget */
		XtWarning("BAD widget id in XwHistbarchangecolor");
		return;
	}

	if(ID >= cw->histbar.nactc || ID < 0) 
		return;
 	colorpix[0] = cw->histbar.bars[ID].color;
	colorpix[1] = cw->histbar.bars[ID].shadecolor;	
	XFreeColors(XtDisplay(w), DefaultColormap(XtDisplay(w)
		,DefaultScreen(XtDisplay(w))),colorpix,2,(int)NULL);		
	cw->histbar.bars[ID].color = color;
	Gethistbarcolours (cw , &(cw->histbar.bars[ID]) );
	
}

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		XwHistbarSettext
 *  
 *  Input:			Widget w
 *			int ID 		id of requeseted bar
 *			char *text	set text to display (empty or NULL to rest)
 *
 *  Return:			
 *
 *  Documentation:		sets the per bar text display.
 *			    CAUTION: There is no limit check for number of characters
 *  	    	    	    so check if the text fits with fontsize into one bar
 *
 *
 *  History:		Sun Jan 25 08:05:44 CET 1998
 *
 ***************************************************************************
 ***************************************************************************/
void XwHistbarSettext(Widget w, int ID , char *text){
  XwHistbarWidget cw = (XwHistbarWidget) w;

	if(!cw->histbar.bars || !cw->histbar.gc){	/* indication for bad widget */
		XtWarning("BAD widget id in XwHistbarSettext");
		return;
	}

	if(ID >= cw->histbar.nactc || ID < 0) 
		return;
	if(cw->histbar.bars[ID].fixtext)
		free(cw->histbar.bars[ID].fixtext);
	if(!text || !strlen(text) ){
		cw->histbar.bars[ID].fixtext = NULL;
		return;
	}
	cw->histbar.bars[ID].fixtext = strdup(text);
	if(cw->histbar.barnames && cw->histbar.exposed)
	    drawannot(cw);
		

}

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		XwHistbarAddbar
 *  
 *  Input:			Widget cw
 *			unsigned long int colour
 *
 *  Return:		long int 			Bar ID	
 *
 *  Documentation:		Adds a Bar storage to the widget, returns the ID
 *
 *
 *  History:		Sun Jan 18 08:56:30 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
long int XwHistbarAddbar( Widget w , unsigned long int colour){
  XwHistbarWidget cw = (XwHistbarWidget) w;
  float posx=cw->histbar.pixwidth - cw->histbar.width,posy = cw->histbar.height , last=0;

	if(!cw->histbar.bars || !cw->histbar.gc){	/* indication for bad widget */
		XtWarning("BAD widget id in XwHistbarAddbar");
		return -1;
	}

	if( cw->histbar.nactc >= cw->histbar.nbars) {	/* need to reallocate */
		cw->histbar.nbars += DefaultNbars ;
		cw->histbar.bars = (ScBars *)XtRealloc(
			(char *)cw->histbar.bars ,cw->histbar.nbars * sizeof(ScBars));
		  if(!cw->histbar.bars  ) 
 	 		XtWarning("WHistbar   XwHistbarAddbar   cannot XtRealloc");
 	 	
	}
	memset(&cw->histbar.bars[cw->histbar.nactc] , 0 , sizeof(ScBars));
	cw->histbar.bars[cw->histbar.nactc].color = colour;

	Gethistbarcolours(cw , &(cw->histbar.bars[cw->histbar.nactc]));
	if(cw->histbar.nactc){
		posx = cw->histbar.bars[cw->histbar.nactc-1].foldx;
		posy = cw->histbar.bars[cw->histbar.nactc-1].foldy;
		last = cw->histbar.bars[cw->histbar.nactc-1].lasttim;
	}
	else
		last = cw->histbar.vis[0] ;
	cw->histbar.bars[cw->histbar.nactc].lasttim = last;
	cw->histbar.bars[cw->histbar.nactc].id = cw->histbar.nactc;
	cw->histbar.bars[cw->histbar.nactc].foldx = posx;
	cw->histbar.bars[cw->histbar.nactc].foldy = posy;
	cw->histbar.bars[cw->histbar.nactc].Xpos = (Dimension)(cw->histbar.nactc * cw->histbar.barfactor
		 * cw->histbar.barwidth + cw->histbar.barwidth);
	
	if(cw->histbar.history)
		add_history(cw,cw->histbar.nactc);
	cw->histbar.nactc++;
	return (cw->histbar.nactc -1);	/* return the ID */
		
}

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		XwHistdobar
 *  
 *  Input:			Widget    w 	target histbaring widget
 *			long int ID	id of bar
 *			float xy[]		value pair (x , y )
 *			int push		if 1 pushes histbaring, 
 *			
 *  Return:			
 *
 *  Documentation:		adds a point to the actual bar(connecting to last stored piunt
 *			if either push is true, or only one bar existing
 *			forces histbaring to actual x value
 *
 *
 *  History:		Sun Jan 18 08:56:47 CET 1998 on host: wizant
 *
 *
 ***************************************************************************
 ***************************************************************************/
 void XwHistdobar (Widget    w  , long int ID ,  float xy[] , int push){
   XwHistbarWidget cw = (XwHistbarWidget) w;
   char message[100];

	if(!cw->histbar.bars || !cw->histbar.gc){	/* indication for bad widget */
		XtWarning("BAD widget id in XwHistdobar");
		return;
	}


 /* first check for correct ID, error if bad */
 	if(ID >= cw->histbar.nactc){
 		sprintf(message,"XwHistdobar:   ERROR   given ID: %ld  is not initialized",ID);
 		XtWarning(message);
 		fprintf(stderr,"XwHistdobar:   ERROR   given ID: %ld  is not initialized\n",ID);
 		return;
 	} 
 /*  first check if pixmap overrun will occur, than refresh pixmap */


	drawbar (cw,  ID , xy[1] , xy[0]);

 	cw->histbar.bars[ID].lasttim = xy[0];
	cw->histbar.bars[ID].lasty = xy[1];
	if(cw->histbar.history)
		addhistory(&cw->histbar.bars[ID].history , xy);
/*********	        cw->histbar.lastpush = xy[0];   !!!!!!!!!!!!!!!!!!!!!!!!!*/
    	copypix(cw , (push || cw->histbar.nactc == 1));
 }  
 

/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		XwHistbarhistory
 *  
 *  Input:			Widget    w
 *			long int ID	requested bar ID
 *			
 *  Output		 float **x		pointer to abscissa array
 *			float **y 		pointer to ordinate array
 *			 int *size		usable size of returned array
 *  Return:			
 *
 *  Documentation:	create (malloc)arrays for ordinate and abscissa and returns them
 *			together with the number of point pairs
 *			the user is responsible to free (free() ) the arrays
 *
 *
 *  History:		Sun Jan 18 08:57:05 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
 void XwHistbarhistory (Widget    w  , long int ID ,  float **x , float **y , int *size){ 
    XwHistbarWidget cw = (XwHistbarWidget) w;
    int i=0 , idx;
    float *lx,*ly;

	if(!cw->histbar.bars || !cw->histbar.gc){	/* indication for bad widget */
		XtWarning("BAD widget id in XwHistbarhistory");
		return;
	}

	if( !cw->histbar.history || !cw->histbar.bars[ID].history.size
	    || ID >= cw->histbar.nactc)
		return;
 	lx = (float *)malloc(sizeof(float) * cw->histbar.bars[ID].history.size);
 	ly = (float *)malloc(sizeof(float) * cw->histbar.bars[ID].history.size);
 	 if(!lx || !ly ) 
 	 	XtWarning("WHistbar   XwHistbarhistory  cannot malloc");

 	idx = cw->histbar.bars[ID].history.firstindex;
 	if(idx<0)
 		 idx = 0;
 	do{
 	    lx[i] = cw->histbar.bars[ID].history.tim[idx];
 	    ly[i] = cw->histbar.bars[ID].history.y[idx];
 	    i++;
	    if(++idx >=  cw->histbar.bars[ID].history.size)
 		idx = 0; 	       
 	}while(idx != cw->histbar.bars[ID].history.lastindex);
	*size = i;	
 	*x = lx;
 	*y = ly;
 }	


/***************************************************************************
 ***************************************************************************
 *
 *  Functionname:		XwHistbarGetValue
 *  
 *  Input:			Widget w
 *			 long int ID		ID of checked bar
 *			float x			abscissa value searched
 *
 *  Return:		float		found ordinate value,  999999 in case no history
 *						999999.000000 in case of error
 *
 *  Documentation:	searches the history array and interpolates for given
 *			time the ordinate value
 *  	    	    	if the searchtime is greater than the ladt stored value, 
 *  	    	    	the last y-value , and if the time is less than the latest
 *  	    	    	timevalue in history, the latest y-value is returned
 *
 *
 *  History:		Thu Jan 29 17:52:45 CET 1998
 *
 *
 ***************************************************************************
 ***************************************************************************/
float XwHistbarGetValue(Widget w , long int ID , float x){
    XwHistbarWidget cw = (XwHistbarWidget) w;
    int idx , idx1;
    float lx;

	if(!cw->histbar.bars || !cw->histbar.gc){	/* indication for bad widget */
		XtWarning("BAD widget id in XwHistbarGetValue");
		return 999999;
	}
	if( !cw->histbar.history || !cw->histbar.bars[ID].history.size
	    || ID >= cw->histbar.nactc)
		return 999999;
 	idx = cw->histbar.bars[ID].history.firstindex;
	if(x > cw->histbar.bars[ID].history.tim[cw->histbar.bars[ID].history.lastindex])
	    return cw->histbar.bars[ID].history.y[cw->histbar.bars[ID].history.lastindex];
	if(x < cw->histbar.bars[ID].history.tim[idx])
	    return cw->histbar.bars[ID].history.y[idx];
 	if(idx<0)
 		 idx = 0;
 	lx = cw->histbar.bars[ID].history.tim[idx];	
 	idx1 = idx; 
  	if(++idx >=  cw->histbar.bars[ID].history.size)
 		idx = 0; 	       
	
 	do{
	       if(cw->histbar.bars[ID].history.tim[idx] >= x  && lx <= x){
 		return  (cw->histbar.bars[ID].history.y[idx1] 
 		     +( (cw->histbar.bars[ID].history.y[idx] - cw->histbar.bars[ID].history.y[idx1])
 		     * (x - lx)) / (cw->histbar.bars[ID].history.tim[idx] - lx)) ;
 	       }
 	       idx1=idx;
 	       lx = cw->histbar.bars[ID].history.tim[idx];
 	      if(++idx >=  cw->histbar.bars[ID].history.size)
 		idx = 0; 	       
 	}while(idx != cw->histbar.bars[ID].history.lastindex);
	return 999999.;

}	
#ifdef __cplusplus
 }
#endif
